@@ -416,20 +416,16 @@ _coretemp_init_component( int cidx )
 
      do {
         char *strCpy;
-        strCpy=strncpy(_coretemp_native_events[i].name,t->name,PAPI_MAX_STR_LEN-2);
-        _coretemp_native_events[i].name[PAPI_MAX_STR_LEN-1] = '\0';
+        strCpy=strncpy(_coretemp_native_events[i].name,t->name,PAPI_MAX_STR_LEN);
         if (strCpy == NULL) HANDLE_STRING_ERROR;
 
         strCpy=strncpy(_coretemp_native_events[i].path,t->path,PATH_MAX);
-        _coretemp_native_events[i].path[PATH_MAX-1] = '\0';
         if (strCpy == NULL) HANDLE_STRING_ERROR;
 
-        strCpy=strncpy(_coretemp_native_events[i].units,t->units,PAPI_MIN_STR_LEN-2);
-	    _coretemp_native_events[i].units[PAPI_MIN_STR_LEN-1] = '\0';
+        strCpy=strncpy(_coretemp_native_events[i].units,t->units,PAPI_MIN_STR_LEN);
         if (strCpy == NULL) HANDLE_STRING_ERROR;
 
-        strCpy=strncpy(_coretemp_native_events[i].description,t->description,PAPI_MAX_STR_LEN-2);
-        _coretemp_native_events[i].description[PAPI_MAX_STR_LEN-1] = '\0';
+        strCpy=strncpy(_coretemp_native_events[i].description,t->description,PAPI_MAX_STR_LEN);
         if (strCpy == NULL) HANDLE_STRING_ERROR;
 
 	    _coretemp_native_events[i].stone = 0;
