@@ -18,21 +18,9 @@
 
 #define PAPI_ROCM_MAX_COUNTERS (512)
 
-#define ROCM_EVENTS_OPENED  (0x1)
-#define ROCM_EVENTS_RUNNING (0x2)
-
-typedef struct {
-    char *name;
-    char *descr;
-    unsigned ntv_dev;
-    unsigned ntv_id;
-    int instance;
-} ntv_event_t;
-
-typedef struct ntv_event_table {
-    ntv_event_t *events;
-    unsigned count;
-} ntv_event_table_t;
+#define ROCM_PROFILE_SAMPLING_MODE (0x0)
+#define ROCM_EVENTS_OPENED         (0x1)
+#define ROCM_EVENTS_RUNNING        (0x2)
 
 extern unsigned _rocm_lock;
 
